(* Graph, a terminal time series graphing utility
   Copyright (C) 2007 Eric Stokes, N. Owen Gunden

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or (at
   your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA. *)

type t = {
  mindelay : float;
  title : string;
  rows: int;
  cols: int;
  range_low: float;
  range_high: float;
  green_range_low: float;
  green_range_high: float;
  range_int: bool;
  scale_factor: float;
  y_axis_width: int;
  field: [ `All | `Specific of int ];
  scale: [ `Log of float | `Linear ];
}

let getenv_default v f default = 
  try f (Sys.getenv v) with Not_found -> default

let parse () =
  let anonargs = ref [] in
  let anonfun a = anonargs := a :: !anonargs in
  let usage_msg = Printf.sprintf "%s [options]" Sys.argv.(0) in
  let title = ref "" in
  let mindelay = ref 0. in
  let rows = ref (getenv_default "LINES" int_of_string 24) in
  let cols = ref (getenv_default "COLUMNS" int_of_string 80) in
  let range_low = ref 0. in
  let range_high = ref (float !rows) in
  let green_range_high = ref None in
  let green_range_low = ref None in
  let range_int = ref false in
  let y_axis_width = ref 6 in
  let scale = ref `Linear in
  let field = ref `All in
  let specl = Arg.align [
    ("-title", Arg.Set_string title, "t set the title of the graph (\"\")");
    ("-mindelay", Arg.Set_float mindelay, "f minimum delay between plotting points (extra datapoints are averaged)");
    ("-logplot", Arg.String (fun b -> scale := `Log (float_of_string b)), 
    "x draw a log plot with base x (false)");
    ("-range", Arg.Tuple [Arg.Set_float range_low; Arg.Set_float range_high;],
    Printf.sprintf "r set the limits of the graph (%f %f)" !range_low !range_high);
    ("-int", Arg.Set range_int,
    Printf.sprintf " treat range as ints in the y-axis (%b)" !range_int);
    ("-y-axis-width", Arg.Set_int y_axis_width,
    Printf.sprintf "x width of y_axis (%d)" !y_axis_width);
    ("-term-columns", Arg.Set_int cols, 
    Printf.sprintf "c set the number of columns (%d)" !cols);
    ("-term-rows", Arg.Set_int rows,
    Printf.sprintf "r set the number of rows (%d)" !rows);
    ("-green-range-high", Arg.Float (fun f -> green_range_high := Some f), 
    "x set the upper bound of the green range (halfway to the top)");
    ("-green-range-low", Arg.Float (fun f ->  green_range_low := Some f),
    "x set the lower bound of the green range (the bottom)");
    ("-k", Arg.Int (fun i -> field := `Specific i),
    "f select a specific field in the data (as in sort) (whole line)")
  ] in
  Arg.parse specl anonfun usage_msg;
  match List.rev !anonargs with
  | [] -> 
      { rows = !rows;
        cols = !cols;
        title = !title;
        mindelay = !mindelay;
        range_low = !range_low;
        range_high = !range_high;
        green_range_low = 
          begin match !green_range_low with
          | Some x -> x
          | None -> !range_low
          end;
        green_range_high = 
          begin match !green_range_high with
          | Some x -> x
          | None -> (!range_high -. !range_low) /. 2.
          end;
        range_int = !range_int;
        scale_factor = (!range_high -. !range_low) /. (float !rows);
        y_axis_width = !y_axis_width;
        scale = !scale;
        field = !field }
  | _ -> (Arg.usage specl usage_msg; exit 1)
