(* Graph, a terminal time series graphing utility
   Copyright (C) 2007 N. Owen Gunden

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or (at
   your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA. *)

open ANSITerminal

type pixel_state = {
  character : char;
  styles : style list;
}

type t = {
  rows : int;
  cols : int;
  prev_screen : pixel_state array array;
  new_screen : pixel_state array array;
}

let check_bounds t ~row ~col msg =
  if row > t.rows - 1 || row < 0 then
    failwith (Printf.sprintf "Termdraw: row %d out of bounds: %s" row msg);
  if col > t.cols - 1 || col < 0 then
    failwith (Printf.sprintf "Termdraw: col %d out of bounds: %s" col msg)

let set_pixel t chr styles ~row ~col =
  check_bounds t ~row ~col "set_pixel";
  t.new_screen.(row).(col) <- { character=chr; styles=styles; }

let write_string t ~row ~col styles s =
  check_bounds t ~row ~col "write_string";
  check_bounds t ~row ~col:(col + String.length s)
    "write_string: string too long?";
  for c = 0 to String.length s - 1 do
    t.new_screen.(row).(col + c) <- { character = s.[c]; styles = styles; }
  done

let draw_pixel pixel ~row ~col =
  set_cursor col row;
  print_string pixel.styles (String.make 1 pixel.character)

let fill t chr styles =
  for row = 0 to t.rows - 1 do
    for col = 0 to t.cols - 1 do
      t.new_screen.(row).(col) <- { character = chr; styles = styles; }
    done
  done

let redraw t =
  for row = 0 to t.rows - 1 do
    for col = 0 to t.cols - 1 do
      if t.prev_screen.(row).(col) <> t.new_screen.(row).(col) then
        begin
          t.prev_screen.(row).(col) <- t.new_screen.(row).(col);
          draw_pixel t.prev_screen.(row).(col) ~row:(row + 1) ~col:(col + 1)
        end
    done
  done;
  flush_all ()

let build_screen ~rows ~cols ~init_styles ~init_character =
  Array.init rows
    (fun row ->
      Array.init cols
        (fun col -> {
          character = init_character;
          styles = init_styles;
        })
    )

let create ~rows ~cols ~init_styles ~init_character = {
    rows = rows;
    cols = cols;
    prev_screen = build_screen ~rows ~cols ~init_styles ~init_character:'0';
    new_screen = build_screen ~rows ~cols ~init_styles ~init_character;
  }
